load scan_parameters.mat
load oneSemsTransfer.mat

%first find the minimum time for which we can assume that all particles
%experienced an exponential voltage change:

tmin=maxtr+td+2*taus; %max residence time + delay in tubes and CPC + 2 * CPC time constant
tmax=min(t_scan+td,t_scan+t_retrace);

firstUsedRawCount=find(tvecRaw>tmin,1)

t=tvecRaw(tvecRaw>tmin & tvecRaw<=tmax);

%Number of counts in raw data between tmin and tmax
nCounts=length(t);

%number of counting intervals used for the inversion
nCountInt=23;
%number of raw counts to be summed up for each counting interval
nRawCountInt=floor(nCounts/nCountInt);
%duration of count interval
durationCountInt=nRawCountInt/scan_resolution; %s
% sampled volume during counting interval
volumeCountInt=durationCountInt*Qa;

%the follwing number should preferably be small.
unusedRawCount=nCounts-nCountInt*nRawCountInt;

%% Now calculate transfer function for the first counting interval:
Zvec=Zvec*exp(-(t(1)-tab)/tau);%Now OmegaSems corresponds to Zvec for the first raw count

%for the following length(Zvec) should be clearly greater than nRawCountInt
%If the scan resolution was not used for the arrival time transfer function
%calculation, the following needs to be modified
OmegaSum=zeros(size(OmegaSems));
for ii=0:nRawCountInt-1
    OmegaSum=OmegaSum+[OmegaSems(1+ii:end) zeros(1,ii)];
end
OmegaCountInt=OmegaSum/nRawCountInt;
figure
plot(Zvec,OmegaCountInt,Zvec,OmegaSems)

%% Loop over counting intervals and calculate kernel
% the shape of the transfer function is the same for all counting
% intervals (ignoring diffusion in the transfer function calculation), so
% rather than changing OmegaCountInt we can change Zvec.

ZvecFirst=Zvec;
for jj=1:nCountInt
    Zvec=ZvecFirst*exp(-(jj-1)*nRawCountInt/scan_resolution/tau);
    
    
    
    %figure
    for ii=1:6
        dp(:,ii)=dpFromMob(Zvec/ii,lambda,visc)';
    %    semilogx(dp(:,ii),OmegaCountInt)
    %    hold on
    end
    
    %tube lengths for diffusion loss calculations
    inletTube=2; %m
    eqTubeLengthClassifier=2.1+1.51+0.33+0.25+9.1; %m
    eqTubeLength=eqTubeLengthClassifier+tubeDMA2CPC+inletTube; %m
    
    for ii=1:6
        chargeFraction=varaus(dp(:,ii),1:6,T);
        tubePenetrate=ltubefl(dp(:,ii),eqTubeLength,Qa,T,visc,lambda)';
        kernel(:,ii)=OmegaCountInt'.*chargeFraction(:,ii).*tubePenetrate;
        [dummy,indexPeak]=max(kernel(:,ii));
        dpPeak(jj,ii)=dp(indexPeak,ii);
        % integrate on logarithmic scale:
        intPeak(jj,ii)=-kernel(2:end,ii)'*diff(log10(dp(:,ii)));
    end
    kernel_record(jj,:,:)=kernel;
   
end
intPeakRel=intPeak./repmat(intPeak(:,1),1,6);
usePeak=intPeakRel>1e-3 & dpPeak<D50/1e9;
%find midpoints of counting intervals;
tCountInt=mean(reshape(t(1:nCountInt*nRawCountInt),nRawCountInt,nCountInt))';

durationCountInt/mean(diff(tCountInt)) %should be one

tCountInt=repmat(tCountInt,1,6)';
dpPeak=dpPeak';

xOneScan=[tCountInt(usePeak')/(24*60*60),log10(dpPeak(usePeak'))];

TOneScan=zeros(sum(sum(usePeak)),nCountInt);
%make kernel matrix
counter=1;
for i=1:nCountInt
    for j=1:6
    if usePeak(i,j)
        TOneScan(counter,i)=intPeak(i,j);
        counter=counter+1;
    end
    end
end



clear dummy chargeFraction indexPeak dp Zvec ZvecFirst OmegaSum OmegaCountInt
clear nRawCount t tvecRaw IntPeakRel usePeak i j ii jj counter
clear intPeak dpPeak


