function [r,z,t]=transferDD2011_v2_2()
% Calculation of transfer function according to Dubey
% and Dhaniyala 2011. Diffusion correction not included.

%In version 2 ATFcontribution has been optimised
%In version 2.1 the parameters are loaded from a file, and td is taken into
%account
% version 2.2 simplifies the code and calculates the SEMS ATF based on the 
% more accurate DMA ATF (only small difference for up-scans).

global tau L r1 r2 ra rb Z Qsh Qa V_beginscan A B C 

% L=0.4444;%m, DMA length
% r1=0.00937;%m
% r2=0.01958;%m
% 
% V_beginscan=10; %V, 
% V_endscan=10000; %V
% 
% %flows
% Qsh=2.5/60/1000; %m3/s
% Qa=0.25/60/1000; %m3/s
% % 
% t_scan=45; %s
% scan_resolution=50; %Hz
% detector=3787; % CPC model
% lambda=6.7e-8; % mean free path
% visc=1.83e-5; %viscosity
% td=1; delay time


% L=0.109;%m, DMA length
% r1=0.025;%m
% r2=0.033;%m
% 
% V_beginscan=10000; %V, volgate at begining of scan
% V_endscan=10; %V

% %flows
% Qsh=5/60/1000; %m3/s
% Qa=1/60/1000; %m3/s

load scan_parameters.mat

% A in Dubey and Dhaniyala (2008) eq 4 obtained integrating the flow over
% the cross section of the DMA.
A=-2/(r2^2+r1^2-(r2^2-r1^2)/log(r2/r1))*(Qsh+Qa)/(pi*(r2^2-r1^2));
B=-A*(r2^2-r1^2)/log(r2/r1);
C=A*((r2^2-r1^2)/log(r2/r1)*log(r1)-r1^2);

%iterate rb:
rb=1.2*r1;

a=0.25;
c=r1^4/4-(r2^2-r1^2)/log(r2/r1)*r1^2/4-Qa/(2*pi*A);
for j=1:150
    rb_old=rb;
    b=(r2^2-r1^2)/log(r2/r1)*(-log(rb_old/r1)/2+0.25)-r1^2/2;
    rb_guess=sqrt((-b+sqrt(b^2-4*a*c))/(2*a));%produces a guess which is further from true rb than rb_old
    rb=2*rb_old-rb_guess;

end

%iterate ra, simply by replacing Qa by Qsh in the calculation of c in the above method for rb
ra=0.9*r2;
A=-2/(r2^2+r1^2-(r2^2-r1^2)/log(r2/r1))*(Qsh+Qa)/(pi*(r2^2-r1^2));
a=0.25;
c=r1^4/4-(r2^2-r1^2)/log(r2/r1)*r1^2/4-Qsh/(2*pi*A);
for j=1:150
    ra_old=ra;
    b=(r2^2-r1^2)/log(r2/r1)*(-log(ra_old/r1)/2+0.25)-r1^2/2;
    ra_guess=sqrt((-b+sqrt(b^2-4*a*c))/(2*a));%produces a guess which is further from true ra than ra_old
    ra=2*ra_old-ra_guess;

end


tau=t_scan/(log(V_endscan/V_beginscan));
Z=1e-7; %m2/(V*s) electric mobility, somewhat arbitrary choice



%Step 1
%iterate to obtain initial and residence times for trajectories between extrema.
[tr_ab,ti_ab,z]=iteratet(ra,rb);
[tr_2b,ti_2b,z]=iteratet(r2,rb);
[tr_a1,ti_a1,z]=iteratet(ra,r1);
[tr_21,ti_21,z]=iteratet(r2,r1);
tab=tr_ab+ti_ab;
t2b=tr_2b+ti_2b;
ta1=tr_a1+ti_a1;
t21=tr_21+ti_21;

tr=mean([tr_ab; tr_2b; tr_a1; tr_21]);


%check whether tab really is the peak time:
nri=300; %number of inlet locations
stepr=(r2-ra)/nri;
ri=(ra+stepr/2:stepr:r2)';

v=A*ri.^2+B*log(ri)+C; %velocities in z direction
w=v.*ri/sum(v.*ri); %weights
sample_tr=[];
sample_t=[];
sample_w=[];

DeltatInject=0.001;
for ti=min([ti_ab; ti_2b; ti_a1; ti_21]):DeltatInject:max([ti_ab; ti_2b; ti_a1; ti_21])+0.6
    ti
    [isSampled,tr_sampled,t_sampled]=ATFcontribution(ti,ri);
    sample_tr=[sample_tr; tr_sampled(isSampled)]; % residence times in DMA
    sample_t=[sample_t; t_sampled(isSampled)]; % times reaching DMA exit slit
    sample_w=[sample_w; w(isSampled)]; %weights (see Dubey and Dhaniyala 2008 eq 15)
    %save sample.mat sample*
    
end
maxtr=max(sample_tr);
save sample.mat sample*

%Create time vector
Deltatc=1/scan_resolution;%counting interval length (s)
tvec=tab-4:Deltatc:tab+10;% vector has to be long enough

%Now sort the sample created in the loop above based on the DMA exit times
%and sum up contributions to get the ATF as a function of tvec
[t_sorted,I]=sort(sample_t);
nelements=hist(t_sorted,tvec);
w_sorted=sample_w(I);
counter=0;
ATFdma=zeros(size(tvec));
for i=1:length(nelements)
    ATFdma(i)=sum(w_sorted(counter+(1:nelements(i))))/(sum(w)*Deltatc/DeltatInject);
    counter=counter+nelements(i); 
end
figure(11)
plot(tvec,ATFdma) 

%step 2
% calculate area directly from the ATF obtained above (rather than using eq 15)
if V_beginscan<V_endscan % only for up-scan
Area=sum(ATFdma)*Deltatc;
%Now set the idealised DMA ATF:
%tvec=tab-4:Deltatc:tab+10;
height=2*Area/(t21-tab);
ATFdma_approx=max(0,min(height*(tvec-t21)/(t2b-t21),height*(tvec-tab)/(t2b-tab)));
figure(11)
hold all
plot(tvec,ATFdma_approx)
end

%step 6
%taus=0.83;%CPC time constant
if detector==3787
    taus=0.13;%CPC time constant
end
denom16=sum(exp(-(0:1000)'*Deltatc/taus));%denominator in equation16 2011 article
ATFsems=zeros(size(tvec));
for j=1:length(tvec)
ATFsems(j)=sum(ATFdma(1:j).*exp(-(tvec(j)-tvec(1:j))/taus))/denom16;
end
tvec=tvec+td-taus/2;%Assuming that td is the average delay
figure(11)
hold all
plot(tvec,ATFsems)

%step 7:
%Ignoring the diffusion all SEMS ATFs look the same. If diffusion is taken
%into account, SEMS ATFs for all mobilities (in steps of exp(Deltatc/tau)) need to be calculated. Put them
%all into a matrix, each row being an ATF for some mobility, then the
%columns will be mobility based transfer functions. In the calculations,
%use counting interval Deltatc short enough. Later on you can calculate
%mean transfer functions for longer counting intervals.

%Now let us calculate the transfer function for some counting interval, say the one
%centered at tab (ignoring diffusion).
Zvec=Z*exp((tvec-tab)/tau);
OmegaSems=ATFsems;
figure(12)
plot(Zvec,OmegaSems)

save oneSemsTransfer.mat tau tab taus Zvec OmegaSems maxtr


dpvec=min_mob(Zvec,lambda,visc);
figure(13)
semilogx(dpvec,OmegaSems)
save
end

function [isSampled,tr_sampled,t_sampled]...
    =ATFcontribution(ti,ri)
global tau V_beginscan L r1 r2 rb A B C Z
%ri is a vector of input radii, ti is a scalar
Vi=V_beginscan*exp(ti/tau);
%residence times based on radii
t1=gettr(ri,Vi,r1);
tb=gettr(ri,Vi,rb);
%axial location after residence time
z1=getz(ri,Vi,t1);
zb=getz(ri,Vi,tb);
vzb=A*rb.^2+B*log(rb)+C;

tr_sampled=nan(size(ri));
isSampled=zb<L & L<z1;
if any(isSampled)
    for ii=find(isSampled)'
        %tvec=tb(ii):0.01:t1(ii)+0.01;
        %zvec=getz(ri(ii),Vi,tvec);
        %index= find(zvec>L, 1 );
        %tr_sampled(ii)=tvec(index)-0.005;
        
        %alternative to four previous lines:
        %Let's find the time spent after reaching rb
        z=zb(ii);
        tOld=tb(ii);
        vzOld=vzb;
        error=1e10;
        while abs(error)>1e-3;
            Deltat=(L-z)/vzOld;%first guess
            r=sqrt(ri(ii)^2-2*Z*Vi*tau/log(r2/r1)*(exp((tOld+Deltat)/tau)-1));
            vz=A*r.^2+B*log(r)+C;
            Deltat=(L-z)/(vzOld+vz)*2;%second guess
            r=sqrt(ri(ii)^2-2*Z*Vi*tau/log(r2/r1)*(exp((tOld+Deltat)/tau)-1));
            vz=A*r.^2+B*log(r)+C;
            z=getz(ri(ii),Vi,tOld+Deltat);
            error=-(L-z)/vz;
            vzOld=vz;
            tOld=tOld+Deltat;
        end
        tr_sampled(ii)=tOld-error;
    end
end
t_sampled=tr_sampled+ti;
end

function [tr,ti,z]=iteratet(ri,rf)
global tau V_beginscan L
z=10*sign(tau);%any big number works here
ti=0;
while isreal(z) && z*sign(tau)>L*sign(tau)
    %When ramping down the voltage, complex tr and z may occur, if the
    %voltage is too small for the particle ever to reach rf. Thus the
    %requirement isreal(z) is used above.
    
    Vi=V_beginscan*exp(ti/tau);
    tr=gettr(ri,Vi,rf);
    z=getz(ri,Vi,tr);
    ti=ti+10;
end
z=10*sign(tau);
ti=ti-20;
while isreal(z) && z*sign(tau)>L*sign(tau)
    
    Vi=V_beginscan*exp(ti/tau);
    tr=gettr(ri,Vi,rf);
    z=getz(ri,Vi,tr);
    ti=ti+1;
end
z=10*sign(tau);
ti=ti-2;

while isreal(z) && z*sign(tau)>L*sign(tau)
    
    Vi=V_beginscan*exp(ti/tau);
    tr=gettr(ri,Vi,rf);
    z=getz(ri,Vi,tr);
    ti=ti+0.1;
end
z=10*sign(tau);
ti=ti-0.2;
while isreal(z) && z*sign(tau)>L*sign(tau)
    
    Vi=V_beginscan*exp(ti/tau);
    tr=gettr(ri,Vi,rf);
    z=getz(ri,Vi,tr);
    ti=ti+0.01;
end
ti=ti-0.02;

while isreal(z) && z*sign(tau)>L*sign(tau)
    
    Vi=V_beginscan*exp(ti/tau);
    tr=gettr(ri,Vi,rf);
    z=getz(ri,Vi,tr);
    ti=ti+0.001;
end
ti=ti-0.0015;
end

function tr=gettr(ri,Vi,rf)
    %calculates residence neeeded to obtain some final radial location rf
    global tau L r1 r2 ra rb Z
    tr=tau*log((ri.^2-rf^2)/(2*Z*Vi*tau)*log(r2/r1)+1);
end

function z=getz(ri,Vi,t)
    %calculates z coordinate for some initial conditions and resedense time
    %t.
    global tau r1 r2 Z Qsh Qa A B C 

    %The constant A is obtained by integration of the flow over a cross
    %section of the DMA. In that way a relation between A and the total
    %flow (Qsh+Qa) is obtained.
    %A=-2/(r2^2+r1^2-(r2^2-r1^2)/log(r2/r1))*(Qsh+Qa)/(pi*(r2^2-r1^2));
    %B=-A*(r2^2-r1^2)/log(r2/r1);
    %C=A*((r2^2-r1^2)/log(r2/r1)*log(r1)-r1^2);
    alpha=2*Z*Vi*tau/log(r2/r1);
    k=ri.^2+alpha;
    z=nan(size(k));
    kpos=k>0;
    kneg=k<0;
    if any(kpos)>0
    
        D=B/2*dilog(alpha./k(kpos));
    
    

        z(kpos)=A*k(kpos).*t(kpos)-A*alpha*tau*(exp(t(kpos)/tau)-1)+B/2*t(kpos).*log(k(kpos))...
            -B*tau/2*dilog(alpha*exp(t(kpos)/tau)./k(kpos))+C*t(kpos)+D*tau;
    end
    if any(kneg)
        yt=alpha./k(kneg).*exp(t(kneg)/tau);
        y0=alpha./k(kneg);

        z(kneg)=A*k(kneg).*t(kneg)-A*alpha*tau*(exp(t(kneg)/tau)-1)+B/2*t(kneg).*log(-k(kneg))...
            +B*tau/2*(log(yt-1).*log(yt)-log(y0-1).*log(y0)+dilog(1-yt)-dilog(1-y0))...
            +C*t(kneg);
    end
    
        

end

function d = dilog(z)
% DILOG = di-Logarithm.
%
% d = dilog(z) = Li_2(z) 
%   = -Int From t=0 To t=z    log(1-t) dt/t         for all z.
%   =  Sum From n=1 To n=Inf  z^n/n^2               for |z|<=1.
%
% INPUT  z: real or complex, scalar, vector or matrix.
% OUTPUT d: component-wise dilogarithm of z.

% References:
% [1] Lewin, L. 1958. Dilogarithms and associated functions. Macdonald.
% [2] Wood, D. C. 1992. Technical Report 15-92. University of Kent computing laboratory.
% [3] http://en.wikipedia.org/wiki/Polylog

% Didier Clamond, February 28th, 2006.

% Initialization.
d  = zeros(size(z));     
s  =  ones(size(z));     
      
% For large moduli: Mapping onto the unit circle |z|<=1.
j = find(abs(z)>1);
d(j) = -1.64493406684822643 - 0.5*log(-z(j)).^2; 
s(j) = -s(j);
z(j) = 1./z(j); 

% For large positive real parts: Mapping onto the unit circle with Re(z)<=1/2.
j = find(real(z)>0.5);
d(j) = d(j) + s(j).*( 1.64493406684822643 - log((1-z(j)).^log(z(j))) );
s(j) = -s(j);
z(j) = 1 - z(j);

% Transformation to Debye function and rational approximation.
z = -log(1-z);                                                                                
s = s.*z;                                                                                      
d = d - 0.25*s.*z;                                                                            
z = z.*z;                                                                                     
s = s.*(1+z.*(6.3710458848408100e-2+z.*(1.04089578261587314e-3+z*4.0481119635180974e-6)));    
s = s./(1+z.*(3.5932681070630322e-2+z.*(3.20543530653919745e-4+z*4.0131343133751755e-7)));    
d = d + s;                                                                                    

end

function tulos=min_mob(mob,lambda,visc)

%Calculate diameter from mobility

dp=1e-9*ones(size(mob));
dpt=ones(size(mob));

e=1.602e-19;
while max(max(abs(dp-dpt) ./dpt)) > 1e-6
dp=dpt;
dpt=e*cunn(dp,lambda) ./(3*pi*visc*mob);
end

tulos=dpt;
end

function c=cunn(dp,lambda)
%Cunningham correction

c=1.0+ lambda ./dp .*(2.514+0.800*exp(-0.55*dp/lambda));
end
